@@ -2,17 +2,22 @@
  * diff3.c :  routines for doing diffs
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ *    Licensed to the Apache Software Foundation (ASF) under one
+ *    or more contributor license agreements.  See the NOTICE file
+ *    distributed with this work for additional information
+ *    regarding copyright ownership.  The ASF licenses this file
+ *    to you under the Apache License, Version 2.0 (the
+ *    "License"); you may not use this file except in compliance
+ *    with the License.  You may obtain a copy of the License at
  *
- * This software is licensed as described in the file COPYING, which
- * you should have received as part of this distribution.  The terms
- * are also available at http://subversion.tigris.org/license-1.html.
- * If newer versions of this license are posted there, you may use a
- * newer version instead, at your option.
+ *      http://www.apache.org/licenses/LICENSE-2.0
  *
- * This software consists of voluntary contributions made by many
- * individuals.  For exact contribution history, see the revision
- * history and logs, available at http://subversion.tigris.org/.
+ *    Unless required by applicable law or agreed to in writing,
+ *    software distributed under the License is distributed on an
+ *    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *    KIND, either express or implied.  See the License for the
+ *    specific language governing permissions and limitations
+ *    under the License.
  * ====================================================================
  */
 
@@ -24,6 +29,7 @@
 #include "svn_pools.h"
 #include "svn_error.h"
 #include "svn_diff.h"
+#include "svn_sorts.h"
 #include "svn_types.h"
 
 #include "diff.h"
@@ -33,221 +39,233 @@ void
 svn_diff__resolve_conflict(svn_diff_t *hunk,
                            svn_diff__position_t **position_list1,
                            svn_diff__position_t **position_list2,
+                           svn_diff__token_index_t num_tokens,
                            apr_pool_t *pool)
 {
-    apr_off_t modified_start = hunk->modified_start + 1;
-    apr_off_t latest_start = hunk->latest_start + 1;
-    apr_off_t common_length;
-    apr_off_t modified_length = hunk->modified_length;
-    apr_off_t latest_length = hunk->latest_length;
-    svn_diff__position_t *start_position[2];
-    svn_diff__position_t *position[2];
-    svn_diff__lcs_t *lcs = NULL;
-    svn_diff__lcs_t **lcs_ref = &lcs;
-    svn_diff_t **diff_ref = &hunk->resolved_diff;
-    apr_pool_t *subpool;
+  apr_off_t modified_start = hunk->modified_start + 1;
+  apr_off_t latest_start = hunk->latest_start + 1;
+  apr_off_t common_length;
+  apr_off_t modified_length = hunk->modified_length;
+  apr_off_t latest_length = hunk->latest_length;
+  svn_diff__position_t *start_position[2];
+  svn_diff__position_t *position[2];
+  svn_diff__token_index_t *token_counts[2];
+  svn_diff__lcs_t *lcs = NULL;
+  svn_diff__lcs_t **lcs_ref = &lcs;
+  svn_diff_t **diff_ref = &hunk->resolved_diff;
+  apr_pool_t *subpool;
 
-    /* First find the starting positions for the
-     * comparison
-     */
+  /* First find the starting positions for the
+   * comparison
+   */
 
-    start_position[0] = *position_list1;
-    start_position[1] = *position_list2;
+  start_position[0] = *position_list1;
+  start_position[1] = *position_list2;
 
-    while (start_position[0]->offset < modified_start)
-      start_position[0] = start_position[0]->next;
+  while (start_position[0]->offset < modified_start)
+    start_position[0] = start_position[0]->next;
 
-    while (start_position[1]->offset < latest_start)
-      start_position[1] = start_position[1]->next;
+  while (start_position[1]->offset < latest_start)
+    start_position[1] = start_position[1]->next;
 
-    position[0] = start_position[0];
-    position[1] = start_position[1];
+  position[0] = start_position[0];
+  position[1] = start_position[1];
 
-    common_length = modified_length < latest_length
-                  ? modified_length : latest_length;
+  common_length = modified_length < latest_length
+                ? modified_length : latest_length;
 
-    while (common_length > 0
-           && position[0]->node == position[1]->node)
-      {
+  while (common_length > 0
+         && position[0]->token_index == position[1]->token_index)
+    {
+      position[0] = position[0]->next;
+      position[1] = position[1]->next;
+
+      common_length--;
+    }
+
+  if (common_length == 0
+      && modified_length == latest_length)
+    {
+      hunk->type = svn_diff__type_diff_common;
+      hunk->resolved_diff = NULL;
+
+      *position_list1 = position[0];
+      *position_list2 = position[1];
+
+      return;
+    }
+
+  hunk->type = svn_diff__type_conflict;
+
+  /* ### If we have a conflict we can try to find the
+   * ### common parts in it by getting an lcs between
+   * ### modified (start to start + length) and
+   * ### latest (start to start + length).
+   * ### We use this lcs to create a simple diff.  Only
+   * ### where there is a diff between the two, we have
+   * ### a conflict.
+   * ### This raises a problem; several common diffs and
+   * ### conflicts can occur within the same original
+   * ### block.  This needs some thought.
+   * ###
+   * ### NB: We can use the node _pointers_ to identify
+   * ###     different tokens
+   */
+
+  subpool = svn_pool_create(pool);
+
+  /* Calculate how much of the two sequences was
+   * actually the same.
+   */
+  common_length = (modified_length < latest_length
+                  ? modified_length : latest_length)
+                - common_length;
+
+  /* If there were matching symbols at the start of
+   * both sequences, record that fact.
+   */
+  if (common_length > 0)
+    {
+      lcs = apr_palloc(subpool, sizeof(*lcs));
+      lcs->next = NULL;
+      lcs->position[0] = start_position[0];
+      lcs->position[1] = start_position[1];
+      lcs->length = common_length;
+
+      lcs_ref = &lcs->next;
+    }
+
+  modified_length -= common_length;
+  latest_length -= common_length;
+
+  modified_start = start_position[0]->offset;
+  latest_start = start_position[1]->offset;
+
+  start_position[0] = position[0];
+  start_position[1] = position[1];
+
+  /* Create a new ring for svn_diff__lcs to grok.
+   * We can safely do this given we don't need the
+   * positions we processed anymore.
+   */
+  if (modified_length == 0)
+    {
+      *position_list1 = position[0];
+      position[0] = NULL;
+    }
+  else
+    {
+      while (--modified_length)
         position[0] = position[0]->next;
+
+      *position_list1 = position[0]->next;
+      position[0]->next = start_position[0];
+    }
+
+  if (latest_length == 0)
+    {
+      *position_list2 = position[1];
+      position[1] = NULL;
+    }
+  else
+    {
+      while (--latest_length)
         position[1] = position[1]->next;
 
-        common_length--;
-      }
+      *position_list2 = position[1]->next;
+      position[1]->next = start_position[1];
+    }
 
-    if (common_length == 0
-        && modified_length == latest_length)
-      {
-        hunk->type = svn_diff__type_diff_common;
-        hunk->resolved_diff = NULL;
+  token_counts[0] = svn_diff__get_token_counts(position[0], num_tokens,
+                                               subpool);
+  token_counts[1] = svn_diff__get_token_counts(position[1], num_tokens,
+                                               subpool);
 
-        *position_list1 = position[0];
-        *position_list2 = position[1];
+  *lcs_ref = svn_diff__lcs(position[0], position[1], token_counts[0],
+                           token_counts[1], num_tokens, 0, 0, subpool);
 
-        return;
-      }
+  /* Fix up the EOF lcs element in case one of
+   * the two sequences was NULL.
+   */
+  if ((*lcs_ref)->position[0]->offset == 1)
+    (*lcs_ref)->position[0] = *position_list1;
 
-    hunk->type = svn_diff__type_conflict;
+  if ((*lcs_ref)->position[1]->offset == 1)
+    (*lcs_ref)->position[1] = *position_list2;
 
-    /* ### If we have a conflict we can try to find the
-     * ### common parts in it by getting an lcs between
-     * ### modified (start to start + length) and
-     * ### latest (start to start + length).
-     * ### We use this lcs to create a simple diff.  Only
-     * ### where there is a diff between the two, we have
-     * ### a conflict.
-     * ### This raises a problem; several common diffs and
-     * ### conflicts can occur within the same original
-     * ### block.  This needs some thought.
-     * ###
-     * ### NB: We can use the node _pointers_ to identify
-     * ###     different tokens
-     */
+  /* Produce the resolved diff */
+  while (1)
+    {
+      if (modified_start < lcs->position[0]->offset
+          || latest_start < lcs->position[1]->offset)
+        {
+          (*diff_ref) = apr_palloc(pool, sizeof(**diff_ref));
 
-    subpool = svn_pool_create(pool);
+          (*diff_ref)->type = svn_diff__type_conflict;
+          (*diff_ref)->original_start = hunk->original_start;
+          (*diff_ref)->original_length = hunk->original_length;
+          (*diff_ref)->modified_start = modified_start - 1;
+          (*diff_ref)->modified_length = lcs->position[0]->offset
+                                         - modified_start;
+          (*diff_ref)->latest_start = latest_start - 1;
+          (*diff_ref)->latest_length = lcs->position[1]->offset
+                                       - latest_start;
+          (*diff_ref)->resolved_diff = NULL;
 
-    /* Calculate how much of the two sequences was
-     * actually the same.
-     */
-    common_length = (modified_length < latest_length
-                    ? modified_length : latest_length)
-                  - common_length;
+          diff_ref = &(*diff_ref)->next;
+        }
 
-    /* If there were matching symbols at the start of
-     * both sequences, record that fact.
-     */
-    if (common_length > 0)
-      {
-        lcs = apr_palloc(subpool, sizeof(*lcs));
-        lcs->next = NULL;
-        lcs->position[0] = start_position[0];
-        lcs->position[1] = start_position[1];
-        lcs->length = common_length;
+      /* Detect the EOF */
+      if (lcs->length == 0)
+        break;
 
-        lcs_ref = &lcs->next;
-      }
+      modified_start = lcs->position[0]->offset;
+      latest_start = lcs->position[1]->offset;
 
-    modified_length -= common_length;
-    latest_length -= common_length;
+      (*diff_ref) = apr_palloc(pool, sizeof(**diff_ref));
 
-    modified_start = start_position[0]->offset;
-    latest_start = start_position[1]->offset;
+      (*diff_ref)->type = svn_diff__type_diff_common;
+      (*diff_ref)->original_start = hunk->original_start;
+      (*diff_ref)->original_length = hunk->original_length;
+      (*diff_ref)->modified_start = modified_start - 1;
+      (*diff_ref)->modified_length = lcs->length;
+      (*diff_ref)->latest_start = latest_start - 1;
+      (*diff_ref)->latest_length = lcs->length;
+      (*diff_ref)->resolved_diff = NULL;
 
-    start_position[0] = position[0];
-    start_position[1] = position[1];
+      diff_ref = &(*diff_ref)->next;
 
-    /* Create a new ring for svn_diff__lcs to grok.
-     * We can safely do this given we don't need the
-     * positions we processed anymore.
-     */
-    if (modified_length == 0)
-      {
-        *position_list1 = position[0];
-        position[0] = NULL;
-      }
-    else
-      {
-        while (--modified_length)
-          position[0] = position[0]->next;
+      modified_start += lcs->length;
+      latest_start += lcs->length;
 
-        *position_list1 = position[0]->next;
-        position[0]->next = start_position[0];
-      }
+      lcs = lcs->next;
+    }
 
-    if (latest_length == 0)
-      {
-        *position_list2 = position[1];
-        position[1] = NULL;
-      }
-    else
-      {
-        while (--latest_length)
-          position[1] = position[1]->next;
+  *diff_ref = NULL;
 
-        *position_list2 = position[1]->next;
-        position[1]->next = start_position[1];
-      }
-
-    *lcs_ref = svn_diff__lcs(position[0], position[1],
-                             subpool);
-
-    /* Fix up the EOF lcs element in case one of
-     * the two sequences was NULL.
-     */
-    if ((*lcs_ref)->position[0]->offset == 1)
-      (*lcs_ref)->position[0] = *position_list1;
-
-    if ((*lcs_ref)->position[1]->offset == 1)
-      (*lcs_ref)->position[1] = *position_list2;
-
-    /* Restore modified_length and latest_length */
-    modified_length = hunk->modified_length;
-    latest_length = hunk->latest_length;
-
-    /* Produce the resolved diff */
-    while (1)
-      {
-        if (modified_start < lcs->position[0]->offset
-            || latest_start < lcs->position[1]->offset)
-          {
-            (*diff_ref) = apr_palloc(pool, sizeof(**diff_ref));
-
-            (*diff_ref)->type = svn_diff__type_conflict;
-            (*diff_ref)->original_start = hunk->original_start;
-            (*diff_ref)->original_length = hunk->original_length;
-            (*diff_ref)->modified_start = modified_start - 1;
-            (*diff_ref)->modified_length = lcs->position[0]->offset
-                                           - modified_start;
-            (*diff_ref)->latest_start = latest_start - 1;
-            (*diff_ref)->latest_length = lcs->position[1]->offset
-                                         - latest_start;
-
-            diff_ref = &(*diff_ref)->next;
-          }
-
-        /* Detect the EOF */
-        if (lcs->length == 0)
-          break;
-
-        modified_start = lcs->position[0]->offset;
-        latest_start = lcs->position[1]->offset;
-
-        (*diff_ref) = apr_palloc(pool, sizeof(**diff_ref));
-
-        (*diff_ref)->type = svn_diff__type_diff_common;
-        (*diff_ref)->original_start = hunk->original_start;
-        (*diff_ref)->original_length = hunk->original_length;
-        (*diff_ref)->modified_start = modified_start - 1;
-        (*diff_ref)->modified_length = lcs->length;
-        (*diff_ref)->latest_start = latest_start - 1;
-        (*diff_ref)->latest_length = lcs->length;
-
-        diff_ref = &(*diff_ref)->next;
-
-        modified_start += lcs->length;
-        latest_start += lcs->length;
-
-        lcs = lcs->next;
-      }
-
-    *diff_ref = NULL;
-
-    svn_pool_destroy(subpool);
+  svn_pool_destroy(subpool);
 }
 
 
 svn_error_t *
-svn_diff_diff3(svn_diff_t **diff,
-               void *diff_baton,
-               const svn_diff_fns_t *vtable,
-               apr_pool_t *pool)
+svn_diff_diff3_2(svn_diff_t **diff,
+                 void *diff_baton,
+                 const svn_diff_fns2_t *vtable,
+                 apr_pool_t *pool)
 {
   svn_diff__tree_t *tree;
   svn_diff__position_t *position_list[3];
+  svn_diff__token_index_t num_tokens;
+  svn_diff__token_index_t *token_counts[3];
+  svn_diff_datasource_e datasource[] = {svn_diff_datasource_original,
+                                        svn_diff_datasource_modified,
+                                        svn_diff_datasource_latest};
   svn_diff__lcs_t *lcs_om;
   svn_diff__lcs_t *lcs_ol;
   apr_pool_t *subpool;
   apr_pool_t *treepool;
+  apr_off_t prefix_lines = 0;
+  apr_off_t suffix_lines = 0;
 
   *diff = NULL;
 
@@ -256,24 +274,32 @@ svn_diff_diff3(svn_diff_t **diff,
 
   svn_diff__tree_create(&tree, treepool);
 
+  SVN_ERR(vtable->datasources_open(diff_baton, &prefix_lines, &suffix_lines,
+                                   datasource, 3));
+
   SVN_ERR(svn_diff__get_tokens(&position_list[0],
                                tree,
                                diff_baton, vtable,
                                svn_diff_datasource_original,
+                               prefix_lines,
                                subpool));
 
   SVN_ERR(svn_diff__get_tokens(&position_list[1],
                                tree,
                                diff_baton, vtable,
                                svn_diff_datasource_modified,
+                               prefix_lines,
                                subpool));
 
   SVN_ERR(svn_diff__get_tokens(&position_list[2],
                                tree,
                                diff_baton, vtable,
                                svn_diff_datasource_latest,
+                               prefix_lines,
                                subpool));
 
+  num_tokens = svn_diff__get_node_count(tree);
+
   /* Get rid of the tokens, we don't need them to calc the diff */
   if (vtable->token_discard_all != NULL)
     vtable->token_discard_all(diff_baton);
@@ -281,11 +307,20 @@ svn_diff_diff3(svn_diff_t **diff,
   /* We don't need the nodes in the tree either anymore, nor the tree itself */
   svn_pool_destroy(treepool);
 
+  token_counts[0] = svn_diff__get_token_counts(position_list[0], num_tokens,
+                                               subpool);
+  token_counts[1] = svn_diff__get_token_counts(position_list[1], num_tokens,
+                                               subpool);
+  token_counts[2] = svn_diff__get_token_counts(position_list[2], num_tokens,
+                                               subpool);
+
   /* Get the lcs for original-modified and original-latest */
-  lcs_om = svn_diff__lcs(position_list[0], position_list[1],
-                         subpool);
-  lcs_ol = svn_diff__lcs(position_list[0], position_list[2],
-                         subpool);
+  lcs_om = svn_diff__lcs(position_list[0], position_list[1], token_counts[0],
+                         token_counts[1], num_tokens, prefix_lines,
+                         suffix_lines, subpool);
+  lcs_ol = svn_diff__lcs(position_list[0], position_list[2], token_counts[0],
+                         token_counts[2], num_tokens, prefix_lines,
+                         suffix_lines, subpool);
 
   /* Produce a merged diff */
   {
@@ -298,7 +333,6 @@ svn_diff_diff3(svn_diff_t **diff,
     apr_off_t modified_sync;
     apr_off_t latest_sync;
     apr_off_t common_length;
-    apr_off_t original_length;
     apr_off_t modified_length;
     apr_off_t latest_length;
     svn_boolean_t is_modified;
@@ -318,7 +352,7 @@ svn_diff_diff3(svn_diff_t **diff,
       }
     else
       {
-        sentinel_position[0].offset = 1;
+        sentinel_position[0].offset = prefix_lines + 1;
         sentinel_position[0].next = NULL;
         position_list[1] = &sentinel_position[0];
       }
@@ -332,7 +366,7 @@ svn_diff_diff3(svn_diff_t **diff,
       }
     else
       {
-        sentinel_position[1].offset = 1;
+        sentinel_position[1].offset = prefix_lines + 1;
         sentinel_position[1].next = NULL;
         position_list[2] = &sentinel_position[1];
       }
@@ -400,7 +434,6 @@ svn_diff_diff3(svn_diff_t **diff,
 
         if (is_modified || is_latest)
           {
-            original_length = original_sync - original_start;
             modified_length = modified_sync - modified_start;
             latest_length = latest_sync - latest_start;
 
@@ -419,6 +452,7 @@ svn_diff_diff3(svn_diff_t **diff,
                 svn_diff__resolve_conflict(*diff_ref,
                                            &position_list[1],
                                            &position_list[2],
+                                           num_tokens,
                                            pool);
               }
             else if (is_modified)
@@ -441,21 +475,23 @@ svn_diff_diff3(svn_diff_t **diff,
                           - (original_sync - lcs_om->position[0]->offset);
         latest_length = lcs_ol->length
                         - (original_sync - lcs_ol->position[0]->offset);
-        common_length = modified_length < latest_length
-                        ? modified_length : latest_length;
+        common_length = MIN(modified_length, latest_length);
 
-        (*diff_ref) = apr_palloc(pool, sizeof(**diff_ref));
+        if (common_length > 0)
+          {
+            (*diff_ref) = apr_palloc(pool, sizeof(**diff_ref));
 
-        (*diff_ref)->type = svn_diff__type_common;
-        (*diff_ref)->original_start = original_sync - 1;
-        (*diff_ref)->original_length = common_length;
-        (*diff_ref)->modified_start = modified_sync - 1;
-        (*diff_ref)->modified_length = common_length;
-        (*diff_ref)->latest_start = latest_sync - 1;
-        (*diff_ref)->latest_length = common_length;
-        (*diff_ref)->resolved_diff = NULL;
+            (*diff_ref)->type = svn_diff__type_common;
+            (*diff_ref)->original_start = original_sync - 1;
+            (*diff_ref)->original_length = common_length;
+            (*diff_ref)->modified_start = modified_sync - 1;
+            (*diff_ref)->modified_length = common_length;
+            (*diff_ref)->latest_start = latest_sync - 1;
+            (*diff_ref)->latest_length = common_length;
+            (*diff_ref)->resolved_diff = NULL;
 
-        diff_ref = &(*diff_ref)->next;
+            diff_ref = &(*diff_ref)->next;
+          }
 
         /* Set the new offsets */
         original_start = original_sync + common_length;
