@@ -31,9 +31,11 @@
 #include "../../libsvn_client/client.h"
 #include "svn_pools.h"
 #include "svn_client.h"
+#include "private/svn_client_private.h"
 #include "private/svn_client_mtcc.h"
 #include "svn_repos.h"
 #include "svn_subst.h"
+#include "private/svn_sorts_private.h"
 #include "private/svn_wc_private.h"
 #include "svn_props.h"
 #include "svn_hash.h"
@@ -59,7 +61,7 @@ create_greek_repos(const char **repos_url,
   svn_fs_txn_t *txn;
   svn_fs_root_t *txn_root;
 
-  /* Create a filesytem and repository. */
+  /* Create a filesystem and repository. */
   SVN_ERR(svn_test__create_repos(
               &repos, svn_test_data_path(name, pool), opts, pool));
 
@@ -370,7 +372,7 @@ test_patch(const svn_test_opts_t *opts,
     "+It is really the file 'gamma'."
   };
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "test-patch-repos", opts, pool));
 
   /* Check out the HEAD revision */
@@ -445,7 +447,7 @@ test_wc_add_scenarios(const svn_test_opts_t *opts,
   const char *ex_dir_path;
   const char *ex2_dir_path;
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "test-wc-add-repos", opts, pool));
   committed_rev = 1;
 
@@ -574,7 +576,7 @@ test_copy_crash(const svn_test_opts_t *opts,
   const char *dest;
   const char *repos_url;
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "test-copy-crash", opts, pool));
 
   SVN_ERR(svn_client_create_context(&ctx, pool));
@@ -608,7 +610,7 @@ test_16k_add(const svn_test_opts_t *opts,
   apr_pool_t *iterpool = svn_pool_create(pool);
   int i;
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "test-16k-repos", opts, pool));
 
   /* Check out the HEAD revision */
@@ -669,7 +671,7 @@ test_youngest_common_ancestor(const svn_test_opts_t *opts,
   const char *dest;
   svn_client__pathrev_t *yc_ancestor;
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "test-youngest-common-ancestor", opts, pool));
 
   SVN_ERR(svn_client_create_context(&ctx, pool));
@@ -734,20 +736,23 @@ test_foreign_repos_copy(const svn_test_opts_t *opts,
   const char *repos2_url;
   const char *wc_path;
   svn_client_ctx_t *ctx;
-/* Create a filesytem and repository containing the Greek tree. */
+  svn_ra_session_t *ra_session;
+  svn_client__pathrev_t *loc;
+
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "foreign-copy1", opts, pool));
   SVN_ERR(create_greek_repos(&repos2_url, "foreign-copy2", opts, pool));
 
   wc_path = svn_test_data_path("test-foreign-repos-copy", pool);
 
-  wc_path = svn_dirent_join(wc_path, "foreign-wc", pool);
-
   /* Remove old test data from the previous run */
   SVN_ERR(svn_io_remove_dir2(wc_path, TRUE, NULL, NULL, pool));
 
   SVN_ERR(svn_io_make_dir_recursively(wc_path, pool));
   svn_test_add_dir_cleanup(wc_path);
 
+  wc_path = svn_dirent_join(wc_path, "foreign-wc", pool);
+
   rev.kind = svn_opt_revision_head;
   peg_rev.kind = svn_opt_revision_unspecified;
   SVN_ERR(svn_client_create_context(&ctx, pool));
@@ -755,19 +760,26 @@ test_foreign_repos_copy(const svn_test_opts_t *opts,
   SVN_ERR(svn_client_checkout3(NULL, repos_url, wc_path, &peg_rev, &rev,
                                svn_depth_infinity, FALSE, FALSE, ctx, pool));
 
-  SVN_ERR(svn_client__copy_foreign(svn_path_url_add_component2(repos2_url, "A",
-                                                               pool),
-                                   svn_dirent_join(wc_path, "A-copied", pool),
-                                   &peg_rev, &rev, svn_depth_infinity, FALSE, FALSE,
-                                   ctx, pool));
+  SVN_ERR(svn_client__ra_session_from_path2(&ra_session, &loc,
+                                            repos2_url, NULL, &peg_rev, &rev,
+                                            ctx, pool));
 
+  loc->url = svn_path_url_add_component2(repos2_url, "A", pool);
+  SVN_WC__CALL_WITH_WRITE_LOCK(
+    svn_client__repos_to_wc_copy_by_editor(NULL /*sleep*/, svn_node_dir,
+                             loc->url, loc->rev,
+                             svn_dirent_join(wc_path, "A-copied", pool),
+                             ra_session, ctx, pool),
+    ctx->wc_ctx, wc_path, FALSE, pool);
 
-  SVN_ERR(svn_client__copy_foreign(svn_path_url_add_component2(repos2_url,
-                                                               "iota",
-                                                               pool),
-                                   svn_dirent_join(wc_path, "iota-copied", pool),
-                                   &peg_rev, &rev, svn_depth_infinity, FALSE, FALSE,
-                                   ctx, pool));
+  SVN_ERR(svn_ra_reparent(ra_session, repos2_url, pool));
+  loc->url = svn_path_url_add_component2(repos2_url, "iota", pool);
+  SVN_WC__CALL_WITH_WRITE_LOCK(
+    svn_client__repos_to_wc_copy_by_editor(NULL /*sleep*/, svn_node_file,
+                             loc->url, loc->rev,
+                             svn_dirent_join(wc_path, "iota-copied", pool),
+                             ra_session, ctx, pool),
+    ctx->wc_ctx, wc_path, FALSE, pool);
 
   return SVN_NO_ERROR;
 }
@@ -786,7 +798,7 @@ test_suggest_mergesources(const svn_test_opts_t *opts,
 
   peg_rev.kind = svn_opt_revision_unspecified;
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "mergesources", opts, pool));
 
   SVN_ERR(svn_client_create_context(&ctx, pool));
@@ -927,7 +939,7 @@ test_remote_only_status(const svn_test_opts_t *opts, apr_pool_t *pool)
 
   SVN_ERR(svn_stream_mark(contentstream, &start, pool));
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "test-remote-only-status", opts, pool));
 
   SVN_ERR(svn_client_create_context(&ctx, pool));
@@ -953,7 +965,7 @@ test_remote_only_status(const svn_test_opts_t *opts, apr_pool_t *pool)
 
   /* Check out a sparse root @r1 of the repository */
   wc_path = svn_test_data_path("test-remote-only-status-wc", pool);
-  /*svn_test_add_dir_cleanup(wc_path);*/
+  svn_test_add_dir_cleanup(wc_path);
   SVN_ERR(svn_io_remove_dir2(wc_path, TRUE, NULL, NULL, pool));
 
   rev.kind = svn_opt_revision_number;
@@ -1014,8 +1026,7 @@ test_remote_only_status(const svn_test_opts_t *opts, apr_pool_t *pool)
   SVN_TEST_ASSERT(results->nelts == i);
 
   if (opts->verbose)
-    qsort(results->elts, results->nelts, results->elt_size,
-          compare_status_paths);
+    svn_sort__array(results, compare_status_paths);
 
   for (i = 0; i < results->nelts; ++i)
     {
@@ -1096,7 +1107,7 @@ test_copy_pin_externals(const svn_test_opts_t *opts,
     { NULL },
   };
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "pin-externals", opts, pool));
 
   wc_path = svn_test_data_path("pin-externals-working-copy", pool);
@@ -1323,7 +1334,7 @@ test_copy_pin_externals_select_subtree(const svn_test_opts_t *opts, apr_pool_t *
     { NULL },
   };
 
-  /* Create a filesytem and repository containing the Greek tree. */
+  /* Create a filesystem and repository containing the Greek tree. */
   SVN_ERR(create_greek_repos(&repos_url, "pin-externals-select-subtree",
                              opts, pool));
 
